@@ -40,8 +40,6 @@
 
 #include "clutter-build-config.h"
 
-#define CLUTTER_ENABLE_EXPERIMENTAL_API
-
 #include "clutter-backend-private.h"
 #include "clutter-debug.h"
 #include "clutter-event-private.h"
@@ -52,18 +50,8 @@
 #include "clutter-stage-private.h"
 #include "clutter-stage-window.h"
 
-#ifdef CLUTTER_HAS_WAYLAND_COMPOSITOR_SUPPORT
-#include "wayland/clutter-wayland-compositor.h"
-#endif
-
 #include <cogl/cogl.h>
 
-#ifdef CLUTTER_HAS_WAYLAND_COMPOSITOR_SUPPORT
-#include <cogl/cogl-wayland-server.h>
-#include <wayland-server.h>
-#include "wayland/clutter-wayland-compositor.h"
-#endif
-
 #define DEFAULT_FONT_NAME       "Sans 10"
 
 enum
@@ -79,12 +67,6 @@ G_DEFINE_ABSTRACT_TYPE (ClutterBackend, clutter_backend, G_TYPE_OBJECT)
 
 static guint backend_signals[LAST_SIGNAL] = { 0, };
 
-/* Global for being able to specify a compositor side wayland display
- * pointer before clutter initialization */
-#ifdef CLUTTER_HAS_WAYLAND_COMPOSITOR_SUPPORT
-static struct wl_display *_wayland_compositor_display;
-#endif
-
 static void
 clutter_backend_dispose (GObject *gobject)
 {
@@ -93,28 +75,20 @@ clutter_backend_dispose (GObject *gobject)
   /* clear the events still in the queue of the main context */
   _clutter_clear_events_queue ();
 
-  g_clear_pointer (&backend->dummy_onscreen, cogl_object_unref);
+  g_clear_object (&backend->dummy_onscreen);
   if (backend->stage_window)
     {
       g_object_remove_weak_pointer (G_OBJECT (backend->stage_window),
                                     (gpointer *) &backend->stage_window);
+      backend->stage_window = NULL;
     }
 
-  G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
-}
-
-static void
-clutter_backend_finalize (GObject *gobject)
-{
-  ClutterBackend *backend = CLUTTER_BACKEND (gobject);
-
-  g_source_destroy (backend->cogl_source);
-
-  g_free (backend->font_name);
-  clutter_backend_set_font_options (backend, NULL);
+  g_clear_pointer (&backend->cogl_source, g_source_destroy);
+  g_clear_pointer (&backend->font_name, g_free);
+  g_clear_pointer (&backend->font_options, cairo_font_options_destroy);
   g_clear_object (&backend->input_method);
 
-  G_OBJECT_CLASS (clutter_backend_parent_class)->finalize (gobject);
+  G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
 }
 
 static gfloat
@@ -215,22 +189,20 @@ clutter_backend_do_real_create_context (ClutterBackend  *backend,
 {
   ClutterBackendClass *klass;
   CoglSwapChain *swap_chain;
-  GError *internal_error;
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
 
   swap_chain = NULL;
-  internal_error = NULL;
 
   CLUTTER_NOTE (BACKEND, "Creating Cogl renderer");
-  backend->cogl_renderer = klass->get_renderer (backend, &internal_error);
+  backend->cogl_renderer = klass->get_renderer (backend, error);
 
   if (backend->cogl_renderer == NULL)
     goto error;
 
   CLUTTER_NOTE (BACKEND, "Connecting the renderer");
   cogl_renderer_set_driver (backend->cogl_renderer, driver_id);
-  if (!cogl_renderer_connect (backend->cogl_renderer, &internal_error))
+  if (!cogl_renderer_connect (backend->cogl_renderer, error))
     goto error;
 
   CLUTTER_NOTE (BACKEND, "Creating Cogl swap chain");
@@ -242,7 +214,7 @@ clutter_backend_do_real_create_context (ClutterBackend  *backend,
       backend->cogl_display = klass->get_display (backend,
                                                   backend->cogl_renderer,
                                                   swap_chain,
-                                                  &internal_error);
+                                                  error);
     }
   else
     {
@@ -258,7 +230,7 @@ clutter_backend_do_real_create_context (ClutterBackend  *backend,
        */
       res = cogl_renderer_check_onscreen_template (backend->cogl_renderer,
                                                    tmpl,
-                                                   &internal_error);
+                                                   error);
 
       if (!res)
         goto error;
@@ -272,17 +244,12 @@ clutter_backend_do_real_create_context (ClutterBackend  *backend,
   if (backend->cogl_display == NULL)
     goto error;
 
-#ifdef CLUTTER_HAS_WAYLAND_COMPOSITOR_SUPPORT
-  cogl_wayland_display_set_compositor_display (backend->cogl_display,
-                                               _wayland_compositor_display);
-#endif
-
   CLUTTER_NOTE (BACKEND, "Setting up the display");
-  if (!cogl_display_setup (backend->cogl_display, &internal_error))
+  if (!cogl_display_setup (backend->cogl_display, error))
     goto error;
 
   CLUTTER_NOTE (BACKEND, "Creating the Cogl context");
-  backend->cogl_context = cogl_context_new (backend->cogl_display, &internal_error);
+  backend->cogl_context = cogl_context_new (backend->cogl_display, error);
   if (backend->cogl_context == NULL)
     goto error;
 
@@ -446,19 +413,12 @@ _clutter_create_backend (void)
   return retval;
 }
 
-static void
-clutter_backend_real_init_events (ClutterBackend *backend)
-{
-  g_error ("Unknown input backend");
-}
-
 static void
 clutter_backend_class_init (ClutterBackendClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->dispose = clutter_backend_dispose;
-  gobject_class->finalize = clutter_backend_finalize;
 
   /**
    * ClutterBackend::resolution-changed:
@@ -514,7 +474,6 @@ clutter_backend_class_init (ClutterBackendClass *klass)
   klass->resolution_changed = clutter_backend_real_resolution_changed;
   klass->font_changed = clutter_backend_real_font_changed;
 
-  klass->init_events = clutter_backend_real_init_events;
   klass->create_context = clutter_backend_real_create_context;
   klass->get_features = clutter_backend_real_get_features;
 }
@@ -653,17 +612,6 @@ _clutter_backend_get_features (ClutterBackend *backend)
   return 0;
 }
 
-void
-_clutter_backend_init_events (ClutterBackend *backend)
-{
-  ClutterBackendClass *klass;
-
-  g_assert (CLUTTER_IS_BACKEND (backend));
-
-  klass = CLUTTER_BACKEND_GET_CLASS (backend);
-  klass->init_events (backend);
-}
-
 gfloat
 _clutter_backend_get_units_per_em (ClutterBackend       *backend,
                                    PangoFontDescription *font_desc)
@@ -678,31 +626,6 @@ _clutter_backend_get_units_per_em (ClutterBackend       *backend,
   return backend->units_per_em;
 }
 
-void
-_clutter_backend_copy_event_data (ClutterBackend     *backend,
-                                  const ClutterEvent *src,
-                                  ClutterEvent       *dest)
-{
-  ClutterSeatClass *seat_class;
-  ClutterSeat *seat;
-
-  seat = clutter_backend_get_default_seat (backend);
-  seat_class = CLUTTER_SEAT_GET_CLASS (seat);
-  seat_class->copy_event_data (seat, src, dest);
-}
-
-void
-_clutter_backend_free_event_data (ClutterBackend *backend,
-                                  ClutterEvent   *event)
-{
-  ClutterSeatClass *seat_class;
-  ClutterSeat *seat;
-
-  seat = clutter_backend_get_default_seat (backend);
-  seat_class = CLUTTER_SEAT_GET_CLASS (seat);
-  seat_class->free_event_data (seat, event);
-}
-
 /**
  * clutter_get_default_backend:
  *
@@ -872,31 +795,6 @@ clutter_backend_get_cogl_context (ClutterBackend *backend)
   return backend->cogl_context;
 }
 
-#ifdef CLUTTER_HAS_WAYLAND_COMPOSITOR_SUPPORT
-/**
- * clutter_wayland_set_compositor_display:
- * @display: A compositor side struct wl_display pointer
- *
- * This informs Clutter of your compositor side Wayland display
- * object. This must be called before calling clutter_init().
- *
- * Since: 1.8
- * Stability: unstable
- */
-void
-clutter_wayland_set_compositor_display (void *display)
-{
-  if (_clutter_context_is_initialized ())
-    {
-      g_warning ("%s() can only be used before calling clutter_init()",
-                 G_STRFUNC);
-      return;
-    }
-
-  _wayland_compositor_display = display;
-}
-#endif
-
 void
 clutter_set_allowed_drivers (const char *drivers)
 {
@@ -977,3 +875,10 @@ clutter_backend_is_display_server (ClutterBackend *backend)
 {
   return CLUTTER_BACKEND_GET_CLASS (backend)->is_display_server (backend);
 }
+
+void
+clutter_backend_destroy (ClutterBackend *backend)
+{
+  g_object_run_dispose (G_OBJECT (backend));
+  g_object_unref (backend);
+}
