@@ -355,21 +355,10 @@ meta_window_wayland_move_resize_internal (MetaWindow                *window,
         {
           MetaWaylandWindowConfiguration *configuration;
 
-          /* If the constrained size is 1x1 and the unconstrained size is 0x0
-           * it means that we are trying to resize a window where the client has
-           * not yet committed a buffer. The 1x1 constrained size is a result of
-           * how the constraints code works. Lets avoid trying to have the
-           * client configure itself to draw on a 1x1 surface.
-           *
-           * We cannot guard against only an empty unconstrained_rect here,
-           * because the client may have created a xdg surface without a buffer
-           * attached and asked it to be maximized. In such case we should let
-           * it know about the expected window geometry of a maximized window,
-           * even though there is currently no buffer attached. */
-          if (unconstrained_rect.width == 0 &&
-              unconstrained_rect.height == 0 &&
-              constrained_rect.width == 1 &&
-              constrained_rect.height == 1)
+          if (!meta_wayland_surface_get_buffer (window->surface) &&
+              !META_WINDOW_MAXIMIZED (window) &&
+              window->tile_mode == META_TILE_NONE &&
+              !meta_window_is_fullscreen (window))
             return;
 
           configuration =
