@@ -261,6 +261,25 @@ meta_x11_selection_input_stream_close_finish (GInputStream  *stream,
   return TRUE;
 }
 
+static void
+meta_x11_selection_input_stream_dispose (GObject *object)
+{
+  MetaX11SelectionInputStream *stream =
+    META_X11_SELECTION_INPUT_STREAM (object);
+  MetaX11SelectionInputStreamPrivate *priv =
+    meta_x11_selection_input_stream_get_instance_private (stream);
+  MetaX11Display *x11_display;
+
+  x11_display = priv->x11_display;
+  if (x11_display)
+    {
+      x11_display->selection.input_streams =
+        g_list_remove (x11_display->selection.input_streams, stream);
+    }
+
+  G_OBJECT_CLASS (meta_x11_selection_input_stream_parent_class)->dispose (object);
+}
+
 static void
 meta_x11_selection_input_stream_finalize (GObject *object)
 {
@@ -268,6 +287,7 @@ meta_x11_selection_input_stream_finalize (GObject *object)
     META_X11_SELECTION_INPUT_STREAM (object);
   MetaX11SelectionInputStreamPrivate *priv =
     meta_x11_selection_input_stream_get_instance_private (stream);
+  MetaX11Display *x11_display;
 
   g_async_queue_unref (priv->chunks);
 
@@ -275,6 +295,17 @@ meta_x11_selection_input_stream_finalize (GObject *object)
   g_free (priv->target);
   g_free (priv->property);
 
+  x11_display = priv->x11_display;
+  if (x11_display)
+    {
+      Display *xdisplay = meta_x11_display_get_xdisplay (x11_display);
+
+      XDestroyWindow (xdisplay, priv->window);
+
+      g_object_remove_weak_pointer (G_OBJECT (x11_display),
+                                    (gpointer *) &priv->x11_display);
+    }
+
   G_OBJECT_CLASS (meta_x11_selection_input_stream_parent_class)->finalize (object);
 }
 
@@ -284,6 +315,7 @@ meta_x11_selection_input_stream_class_init (MetaX11SelectionInputStreamClass *kl
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GInputStreamClass *istream_class = G_INPUT_STREAM_CLASS (klass);
 
+  object_class->dispose = meta_x11_selection_input_stream_dispose;
   object_class->finalize = meta_x11_selection_input_stream_finalize;
 
   istream_class->read_fn = meta_x11_selection_input_stream_read;
@@ -505,6 +537,9 @@ meta_x11_selection_input_stream_new_async (MetaX11Display      *x11_display,
   priv = meta_x11_selection_input_stream_get_instance_private (stream);
 
   priv->x11_display = x11_display;
+  g_object_add_weak_pointer (G_OBJECT (x11_display),
+                             (gpointer *) &priv->x11_display);
+
   x11_display->selection.input_streams =
     g_list_prepend (x11_display->selection.input_streams, stream);
   priv->selection = g_strdup (selection);
